﻿using System;
using System.Collections.Generic;
using System.Text;
using NeoTicker;
using KreslikLib.Common;
using System.Windows.Forms;

namespace EntryStraddle
{
    public class EntryClass : IDLIndicator
    {
        #region Inputs
        // input names:
        const string LabelSessionEndHour = "Session end - hour";
        const string LabelSessionEndMinute = "Session end - minute";
        const string LabelSessionEndSecond = "Session end - second";
        const string LabelInsideStraddlePips = "Inside straddle pips";
        const string LabelOutsideStraddlePips = "Outside straddle pips";
        const string LabelDataStream = "Data stream";
        // input defaults:
        const int DefSessionEndHour = 3;
        const int DefSessionEndMinute = 0;
        const int DefSessionEndSecond = 0;
        const int DefInsideStraddlePips = 3;
        const int DefOutsideStraddlePips = 4;
        const int DefDataStream = 1;
        #endregion

        #region Class wide objects
        // inputs values:
        int iSessionEndHour;
        int iSessionEndMinute;
        int iSessionEndSecond;
        int iInsideStraddlePips;
        int iOutsideStraddlePips;
        int iDataStream;

        // working objects:
        int decimals;
        bool success;
        double sessionOpenPrice;
        YearMDHMSec currBarDateTime;
        YearMDHMSec prevBarDateTime;
        #endregion

        public double IDLCallEx(NTIndicatorObjects N)
        {
            if (N.ItSelf().FirstCall)
            {
                Initialize(N);
            }

            GetDecimalCount(N);

            currBarDateTime = GetBarDateTime(N);

            if (NewSession(currBarDateTime, prevBarDateTime))
            {
                sessionOpenPrice = GetSessionOpenPrice(N);
            }

            if (sessionOpenPrice > 0 && decimals > 0)
            {
                N.ItSelf().set_Plot(1, sessionOpenPrice + ((double)iInsideStraddlePips / Math.Pow(10, decimals)));
                N.ItSelf().set_Plot(2, sessionOpenPrice + ((double)iOutsideStraddlePips / Math.Pow(10, decimals)));

                N.ItSelf().set_Plot(3, sessionOpenPrice - ((double)iInsideStraddlePips / Math.Pow(10, decimals)));
                N.ItSelf().set_Plot(4, sessionOpenPrice - ((double)iOutsideStraddlePips / Math.Pow(10, decimals)));

                N.ItSelf().set_Plot(5, sessionOpenPrice);

                success = true;
            }

            prevBarDateTime = currBarDateTime;

            N.ItSelf().SuccessAll = success;
            return -1;
        }

        private void Initialize(NTIndicatorObjects N)
        {
            decimals = 0;
            success = false;
            sessionOpenPrice = 0;

            currBarDateTime = new YearMDHMSec();
            prevBarDateTime = new YearMDHMSec();

            GetInputParams(N);
        }

        private void GetDecimalCount(NTIndicatorObjects N)
        {
            if (decimals == 0) // the decimalCount value has not yet been assigned
            {
                double currentPrice = N.DataSeries().get_Items(iDataStream).get_Close(0); // it can be any arbitrary price from the current data stream

                if (currentPrice > 0) // if the current price is a valid one
                {
                    decimals = currentPrice > 40 ? 2 : 4; // forex pairs that are trading above 40 all have 2 decimals precision
                }
            }
        }

        private double GetSessionOpenPrice(NTIndicatorObjects N)
        {
            return N.DataSeries().get_Items(iDataStream).get_Open(0);
        }

        private bool NewSession(YearMDHMSec CurrBarDateTime, YearMDHMSec PrevBarDateTime)
        {
            return (GetSessionEnd(CurrBarDateTime) != GetSessionEnd(PrevBarDateTime)) && PrevBarDateTime.Year != 0;
        }

        private YearMDHMSec GetSessionEnd(YearMDHMSec BarDateTime)
        {
            int sessionEnd_HourMinSec = iSessionEndHour * 10000 + iSessionEndMinute * 100 + iSessionEndSecond;
            int thisBar_HourMinSec = BarDateTime.Hour * 10000 + BarDateTime.Minute * 100 + BarDateTime.Second;

            if (thisBar_HourMinSec > sessionEnd_HourMinSec) // session end is on the next day
            {
                DateTime thisBarDateTime = new DateTime(BarDateTime.Year, BarDateTime.Month, BarDateTime.Day);
                YearMDHMSec sessionEnd = new YearMDHMSec();
                sessionEnd.CopyFrom(thisBarDateTime.AddDays(1));
                sessionEnd.Hour = iSessionEndHour;
                sessionEnd.Minute = iSessionEndMinute;
                sessionEnd.Second = iSessionEndSecond;

                return sessionEnd;
            }
            else // session end is on this day
            {
                return new YearMDHMSec(BarDateTime.Year, BarDateTime.Month, BarDateTime.Day, iSessionEndHour, iSessionEndMinute, iSessionEndSecond);
            }
        }
	
        private YearMDHMSec GetBarDateTime(NTIndicatorObjects N)
        {
            return new YearMDHMSec(
                N.DataSeries().get_Items(iDataStream).get_Year(0),
                N.DataSeries().get_Items(iDataStream).get_Month(0),
                N.DataSeries().get_Items(iDataStream).get_Day(0),
                N.DataSeries().get_Items(iDataStream).get_Hour(0),
                N.DataSeries().get_Items(iDataStream).get_Minute(0),
                N.DataSeries().get_Items(iDataStream).get_Second(0));
        }

        private void GetInputParams(NTIndicatorObjects N)
        {
            iSessionEndHour = GetSessionEndHourOrDefault(N);
            iSessionEndMinute = GetSessionEndMinuteOrDefault(N);
            iSessionEndSecond = GetSessionEndSecondOrDefault(N);
            iInsideStraddlePips = GetStraddlePipsOrDefault(N, 1, DefInsideStraddlePips);
            iOutsideStraddlePips = GetStraddlePipsOrDefault(N, 2, DefOutsideStraddlePips);
            iDataStream = GetDataStreamOrDefault(N);

        }

        private int GetStraddlePipsOrDefault(NTIndicatorObjects N, int Order, int Default)
        {
            string inputName = Order == 1 ? LabelInsideStraddlePips : LabelOutsideStraddlePips;
            int straddlePipsFromInput = N.Params().get_Items(inputName).Int;

            if (straddlePipsFromInput > 0)
            {
                return straddlePipsFromInput;
            }
            else
            {
                ShowError("Invalid "
                        + (Order == 1 ? "inside " : "outside ")
                        + "straddle value: " + straddlePipsFromInput + "\n"
                        + "Reverting to default: [" + Default + "]");

                return Default;
            }
        }

        private int GetSessionEndSecondOrDefault(NTIndicatorObjects N)
        {
            int sessionEndSecondFromInput = N.Params().get_Items(LabelSessionEndSecond).Int;

            if (sessionEndSecondFromInput >= 0 && sessionEndSecondFromInput < 60)
            {
                return sessionEndSecondFromInput;
            }
            else
            {
                ShowError("Invalid session end second value: " + sessionEndSecondFromInput + "\n"
                        + "Reverting to default: [" + DefSessionEndSecond + "]");

                return DefSessionEndSecond;
            }
        }

        private int GetSessionEndMinuteOrDefault(NTIndicatorObjects N)
        {
            int sessionEndMinuteFromInput = N.Params().get_Items(LabelSessionEndMinute).Int;

            if (sessionEndMinuteFromInput >= 0 && sessionEndMinuteFromInput < 60)
            {
                return sessionEndMinuteFromInput;
            }
            else
            {
                ShowError("Invalid session end minute value: " + sessionEndMinuteFromInput + "\n"
                        + "Reverting to default: [" + DefSessionEndMinute + "]");

                return DefSessionEndMinute;
            }
        }

        private int GetSessionEndHourOrDefault(NTIndicatorObjects N)
        {
            int sessionEndHourFromInput = N.Params().get_Items(LabelSessionEndHour).Int;

            if (sessionEndHourFromInput >= 0 && sessionEndHourFromInput < 24)
            {
                return sessionEndHourFromInput;
            }
            else
            {
                ShowError("Invalid session end hour value: " + sessionEndHourFromInput + "\n"
                        + "Reverting to default: [" + DefSessionEndHour + "]");

                return DefSessionEndHour;
            }
        }

        private int GetDataStreamOrDefault(NTIndicatorObjects N)
        {
            int dataStreamFromInput = N.Params().get_Items(LabelDataStream).Int;
            int dataSeriesCount = N.DataSeries().Count;

            if (dataStreamFromInput > 0 && dataStreamFromInput <= dataSeriesCount)
            {
                return dataStreamFromInput;
            }
            else
            {
                ShowError("Invalid data stream: " + dataStreamFromInput + "\n"
                        + "Reverting to default data stream: [" + DefDataStream + "]");

                return DefDataStream;
            }
        }

        private void ShowError(string ErrorMsg)
        {
            MessageBox.Show(ErrorMsg, "[KreslikTools: EntryStraddle] Error", MessageBoxButtons.OK, MessageBoxIcon.Information);
        }
    }
}